"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.oidcServerRequestHandler = void 0;
const oidc_http_server_pages_1 = require("@mongodb-js/oidc-http-server-pages");
function oidcServerRequestHandler(options, info) {
    const { productDocsLink, productName } = options;
    const { res, result, status } = info;
    res.statusCode = status;
    if (result === 'redirecting') {
        res.setHeader('Location', info.location);
        res.end();
        return;
    }
    res.setHeader('Content-Security-Policy', "default-src 'self'; style-src 'unsafe-inline'");
    res.setHeader('Content-Type', 'text/html; charset=utf-8');
    switch (result) {
        case 'accepted':
            res.end((0, oidc_http_server_pages_1.getStaticPage)(oidc_http_server_pages_1.HttpServerPage.OIDCAcceptedPage, {
                productDocsLink,
                productName,
            }));
            break;
        case 'rejected':
            res.end((0, oidc_http_server_pages_1.getStaticPage)(oidc_http_server_pages_1.HttpServerPage.OIDCErrorPage, {
                productDocsLink,
                productName,
                error: info.error,
                errorDescription: info.errorDescription,
                errorURI: info.errorURI,
            }));
            break;
        default:
            res.end((0, oidc_http_server_pages_1.getStaticPage)(oidc_http_server_pages_1.HttpServerPage.OIDCNotFoundPage, {
                productDocsLink,
                productName,
            }));
            break;
    }
}
exports.oidcServerRequestHandler = oidcServerRequestHandler;
//# sourceMappingURL=handler.js.map