import type { Document } from '@mongosh/service-provider-core';
import type Mongo from './mongo';
import type Database from './database';
import { ShellApiWithMongoClass } from './decorators';
import { asPrintable } from './enums';
import type { CommandResult } from './result';
export type ReplSetMemberConfig = {
    _id: number;
    host: string;
    priority?: number;
    votes?: number;
    arbiterOnly?: boolean;
};
export type ReplSetConfig = {
    version: number;
    _id: string;
    members: ReplSetMemberConfig[];
    protocolVersion: number;
};
export default class ReplicaSet extends ShellApiWithMongoClass {
    _database: Database;
    constructor(database: Database);
    get _mongo(): Mongo;
    initiate(config?: Partial<ReplSetConfig>): Promise<Document>;
    _getConfig(): Promise<ReplSetConfig>;
    config(): Promise<ReplSetConfig>;
    conf(): Promise<ReplSetConfig>;
    reconfig(config: Partial<ReplSetConfig>, options?: {}): Promise<Document>;
    reconfigForPSASet(newMemberIndex: number, config: Partial<ReplSetConfig>, options?: {}): Promise<Document>;
    status(): Promise<Document>;
    isMaster(): Promise<Document>;
    hello(): Promise<Document>;
    printSecondaryReplicationInfo(): Promise<CommandResult>;
    printSlaveReplicationInfo(): never;
    printReplicationInfo(): Promise<CommandResult>;
    add(hostport: string | Partial<ReplSetMemberConfig>, arb?: boolean): Promise<Document>;
    addArb(hostname: string): Promise<Document>;
    remove(hostname: string): Promise<Document>;
    freeze(secs: number): Promise<Document>;
    stepDown(stepdownSecs?: number, catchUpSecs?: number): Promise<Document>;
    syncFrom(host: string): Promise<Document>;
    secondaryOk(): Promise<void>;
    [asPrintable](): string;
    private _emitReplicaSetApiCall;
}
